# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import datetime

from driverng.utils import VT, FS
from driverng.protocol.base import BaseProtocol


class MLLPProtocol(BaseProtocol):
    HL7_ACK = b"""MSH|^~\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5\rMSA|AA|%(ack_id)s|\r"""
    ACK_MESSAGE_TYPE = {
        True: b"ORL^022^ORL_022",
        False: b"ACK^R01^ACK",
    }

    def send_trames(self, trames):
        self.write(trames)
        self.recv_ack()
        return True

    def build_trames(self, message):
        # <vt><payload><fs><cr>
        return b"%c%s\r%c\r" % (
            VT,
            b"\r".join(message.splitlines()),
            FS)

    def mllp_readline(self):
        return self.readline(break_func=lambda d: d and d[-2:] == b"%c\r" % (
            FS,))

    def recv_ack(self):
        # TODO check ack
        self.mllp_readline()

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted = payload.splitlines()[0].split(b"|")
        # TODO: Le ack devrait être généré proprement et par MCA
        date = date or datetime.datetime.now()
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
            b"message_type": self.ACK_MESSAGE_TYPE[self.is_automate],
        }
        return self.HL7_ACK % context

    def recv_trames(self):
        data = self.mllp_readline()
        payload = self.get_payload(data)
        self.write(self.build_trames(self.get_ack_message(payload)))
        return payload

    @staticmethod
    def get_payload(data):
        return data[1:-2]
